import argparse
from pathlib import Path
import logging


logger = logging.getLogger(__name__)


def dummy_function():
    """Dummy Function to test logging inside a function"""
    logger.info(f"Logging Test - Function Call Object Done")



from sklearn.impute import SimpleImputer
import numpy as np
from sklearn.metrics import mean_squared_error
import pandas as pd
import pickle
import argparse
import mlflow
import mlflow.sklearn
import logging

remote_server_uri = "http://0.0.0.0:5000"  # set to your server URI
mlflow.set_tracking_uri(remote_server_uri)

parser = argparse.ArgumentParser()
parser.add_argument(
    "read_data",
    type=str,
    help="Read data",
    nargs="?",
    default="data/processed/strat_test_set.csv",
)
parser.add_argument(
    "read_model_pickle",
    type=str,
    help="Read Model",
    nargs="?",
    default="artifacts/model.sav",
)
parser.add_argument(
    "--log-level",
    default=logging.INFO,
    type=lambda x: getattr(logging, x),
    help="Configure the logging level.",
)
parser.add_argument(
    "--no-console",
    default=True,
    type=bool,
    help="Configure the logging console.",
)
parser.add_argument(
    "saveLogs",
    type=str,
    help="Output dir for data",
    nargs="?",
    default="logs/score.log",
)

p = parser.parse_args()

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
logger = logging.getLogger(__name__)
logger.setLevel(level=p.log_level)
sh = logging.StreamHandler()
file_handler = logging.FileHandler(p.saveLogs)
logger.addHandler(file_handler)
console = p.no_console
if console:
    logger.addHandler(sh)


def score(strat_test_set, final_model, housing):
    imputer = SimpleImputer(strategy="median")
    housing_num = housing.drop(["median_house_value", "ocean_proximity"], axis=1)
    imputer.fit(housing_num)
    X_test = strat_test_set.drop("median_house_value", axis=1)
    y_test = strat_test_set["median_house_value"].copy()

    X_test_num = X_test.drop("ocean_proximity", axis=1)
    X_test_prepared = imputer.transform(X_test_num)
    X_test_prepared = pd.DataFrame(
        X_test_prepared, columns=X_test_num.columns, index=X_test.index
    )
    X_test_prepared["rooms_per_household"] = (
        X_test_prepared["total_rooms"] / X_test_prepared["households"]
    )
    X_test_prepared["bedrooms_per_room"] = (
        X_test_prepared["total_bedrooms"] / X_test_prepared["total_rooms"]
    )
    X_test_prepared["population_per_household"] = (
        X_test_prepared["population"] / X_test_prepared["households"]
    )

    X_test_cat = X_test[["ocean_proximity"]]
    X_test_prepared = X_test_prepared.join(pd.get_dummies(X_test_cat, drop_first=True))

    final_predictions = final_model.predict(X_test_prepared)
    final_mse = mean_squared_error(y_test, final_predictions)
    final_rmse = np.sqrt(final_mse)
    logger.info(f"Score generated")
    return final_rmse


strat_test_set = pd.read_csv(p.read_data)
housing = pd.read_csv("data/raw/housing.csv")
# load the model
loaded_model = pickle.load(open(p.read_model_pickle, "rb"))
final_rmse = score(strat_test_set, loaded_model, housing)
print(final_rmse)

experiment_name = "MLE_TRAINING"
try:
    exp_id = mlflow.create_experiment(name=experiment_name)
except Exception as e:
    exp_id = mlflow.get_experiment_by_name(experiment_name).experiment_id

with mlflow.start_run(experiment_id=exp_id):
    mlflow.log_metric(key="rmse", value=final_rmse)



from setuptools import setup, find_packages

classifiers = [
    "Development Status :: 5 - Production/Stable",
    "Intended Audience :: Education",
    "Operating System :: Microsoft :: Windows :: Windows 10",
    "License :: OSI Approved :: MIT License",
    "Programming Language :: Python :: 3",
]

setup(
    name="HousePredicition",
    version="0.0.1",
    description="House Predicition Notebook",
    long_description=open("README.txt").read() + "\n\n" + open("CHANGELOG.txt").read(),
    url="http://pypi.python.org/pypi/HousePredicition/",
    author="Sahil Jaswal",
    author_email="sj89808@gmail.com",
    license="MIT",
    classifiers=classifiers,
    keywords="prediction",
    packages=find_packages(),
    install_requires=[
        "numpy",
        "pandas",
        "scikit-learn",
        "mlflow[extra]",
        "jupyter",
        "pipfile",
        "tensorflow",
        "keras",
    ],
)



import pandas as pd
from sklearn.impute import SimpleImputer
from sklearn.model_selection import StratifiedShuffleSplit
import numpy as np
import os.path
import argparse
import mlflow
import mlflow.sklearn
import logging

remote_server_uri = "http://0.0.0.0:5000"  # set to your server URI
mlflow.set_tracking_uri(remote_server_uri)

parser = argparse.ArgumentParser()
parser.add_argument(
    "outdir",
    type=str,
    help="Output dir for data",
    nargs="?",
    default="data/processed/",
)
parser.add_argument(
    "--log-level",
    default=logging.INFO,
    type=lambda x: getattr(logging, x),
    help="Configure the logging level.",
)
parser.add_argument(
    "--no-console",
    default=True,
    type=bool,
    help="Configure the logging console.",
)
parser.add_argument(
    "saveLogs",
    type=str,
    help="Output dir for data",
    nargs="?",
    default="logs/ingest_data.log",
)
p = parser.parse_args()

# import data
housing = pd.read_csv("data/raw/housing.csv")

logger = logging.getLogger(__name__)
logger.setLevel(level=p.log_level)
sh = logging.StreamHandler()
file_handler = logging.FileHandler(p.saveLogs)
logger.addHandler(file_handler)
console = p.no_console
if console:
    logger.addHandler(sh)


def data_prepration(housing):
    housing["income_cat"] = pd.cut(
        housing["median_income"],
        bins=[0.0, 1.5, 3.0, 4.5, 6.0, np.inf],
        labels=[1, 2, 3, 4, 5],
    )
    split = StratifiedShuffleSplit(n_splits=1, test_size=0.2, random_state=42)
    for train_index, test_index in split.split(housing, housing["income_cat"]):
        strat_train_set = housing.loc[train_index]
        strat_test_set = housing.loc[test_index]
    for set_ in (strat_train_set, strat_test_set):
        set_.drop("income_cat", axis=1, inplace=True)
    housing = strat_train_set.drop("median_house_value", axis=1)
    housing_labels = strat_train_set["median_house_value"].copy()
    imputer = SimpleImputer(strategy="median")

    housing_num = housing.drop("ocean_proximity", axis=1)

    imputer.fit(housing_num)
    X = imputer.transform(housing_num)
    housing_tr = pd.DataFrame(X, columns=housing_num.columns, index=housing.index)
    housing_tr["rooms_per_household"] = (
        housing_tr["total_rooms"] / housing_tr["households"]
    )
    housing_tr["bedrooms_per_room"] = (
        housing_tr["total_bedrooms"] / housing_tr["total_rooms"]
    )
    housing_tr["population_per_household"] = (
        housing_tr["population"] / housing_tr["households"]
    )
    housing_cat = housing[["ocean_proximity"]]
    housing_prepared = housing_tr.join(pd.get_dummies(housing_cat, drop_first=True))
    logger.info(f"Data Prepared")
    return housing_prepared, housing_labels, strat_test_set


housing_prepared, housing_labels, strat_test_set = data_prepration(housing)

housing_prepared.to_csv(
    os.path.join(p.outdir, "housing_prepared.csv"), header=True, index=False
)
housing_labels.to_csv(
    os.path.join(p.outdir, "housing_labels.csv"), header=True, index=False
)
strat_test_set.to_csv(
    os.path.join(p.outdir, "strat_test_set.csv"), header=True, index=False
)

experiment_name = "MLE_TRAINING"
try:
    exp_id = mlflow.create_experiment(name=experiment_name)
except Exception as e:
    exp_id = mlflow.get_experiment_by_name(experiment_name).experiment_id

with mlflow.start_run(experiment_id=exp_id):
    mlflow.log_artifacts(p.outdir)
    mlflow.log_artifact("data/raw/housing.csv")



from sklearn.ensemble import RandomForestRegressor
from sklearn.model_selection import GridSearchCV
import numpy as np
import pandas as pd
import pickle
import mlflow
import mlflow.sklearn
import argparse
import logging

remote_server_uri = "http://0.0.0.0:5000"  # set to your server URI
mlflow.set_tracking_uri(remote_server_uri)

parser = argparse.ArgumentParser()

parser.add_argument(
    "read_data",
    type=str,
    help="Read data",
    nargs="?",
    default="data/processed/housing_prepared.csv",
)
parser.add_argument(
    "read_labels",
    type=str,
    help="Read labels",
    nargs="?",
    default="data/processed/housing_labels.csv",
)
parser.add_argument(
    "modelFile",
    type=str,
    help="model file",
    nargs="?",
    default="artifacts/model.sav",
)
parser.add_argument(
    "outdir",
    type=str,
    help="Output dir for model",
    nargs="?",
    default="artifacts/",
)
parser.add_argument(
    "--log-level",
    default=logging.INFO,
    type=lambda x: getattr(logging, x),
    help="Configure the logging level.",
)
parser.add_argument(
    "--no-console",
    default=True,
    type=bool,
    help="Configure the logging console.",
)
parser.add_argument(
    "saveLogs",
    type=str,
    help="Output dir for data",
    nargs="?",
    default="logs/train.log",
)

p = parser.parse_args()

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
logger = logging.getLogger(__name__)
logger.setLevel(level=p.log_level)
sh = logging.StreamHandler()
file_handler = logging.FileHandler(p.saveLogs)
logger.addHandler(file_handler)
console = p.no_console
if console:
    logger.addHandler(sh)


def train(housing_prepared, housing_labels):
    param_grid = [
        {"n_estimators": [3, 10, 30], "max_features": [2, 4, 6, 8]},
        {"bootstrap": [False], "n_estimators": [3, 10], "max_features": [2, 3, 4]},
    ]
    forest_reg = RandomForestRegressor(random_state=42)
    grid_search = GridSearchCV(
        forest_reg,
        param_grid,
        cv=5,
        scoring="neg_mean_squared_error",
        return_train_score=True,
    )
    grid_search.fit(housing_prepared, housing_labels.values.ravel())

    grid_search.best_params_
    cvres = grid_search.cv_results_
    for mean_score, params in zip(cvres["mean_test_score"], cvres["params"]):
        print(np.sqrt(-mean_score), params)

    final_model = grid_search.best_estimator_
    logger.info(f"Model created")
    logger.debug(f"Model not created")
    return final_model


housing_prepared = pd.read_csv(p.read_data)
housing_labels = pd.read_csv(p.read_labels)

model = train(housing_prepared, housing_labels)
# save the model
pickle.dump(model, open(p.modelFile, "wb"))

experiment_name = "MLE_TRAINING"
try:
    exp_id = mlflow.create_experiment(name=experiment_name)
except Exception as e:
    exp_id = mlflow.get_experiment_by_name(experiment_name).experiment_id

with mlflow.start_run():
    mlflow.log_artifacts(p.outdir)
